@@ -119,7 +119,7 @@ def _connect_ssh(spec):
     """
     Return ContextService arguments for an SSH connection.
     """
-    if C.HOST_KEY_CHECKING:
+    if spec.host_key_checking():
         check_host_keys = 'enforce'
     else:
         check_host_keys = 'ignore'
@@ -158,6 +158,7 @@ def _connect_ssh(spec):
         }
     }
 
+
 def _connect_buildah(spec):
     """
     Return ContextService arguments for a Buildah connection.
@@ -173,6 +174,7 @@ def _connect_buildah(spec):
         }
     }
 
+
 def _connect_docker(spec):
     """
     Return ContextService arguments for a Docker connection.
@@ -276,6 +278,7 @@ def _connect_podman(spec):
         }
     }
 
+
 def _connect_setns(spec, kind=None):
     """
     Return ContextService arguments for a mitogen_setns connection.
@@ -811,7 +814,7 @@ class Connection(ansible.plugins.connection.ConnectionBase):
 
         self.context = dct['context']
         self.chain = CallChain(self, self.context, pipelined=True)
-        if self._play_context.become:
+        if self.become:
             self.login_context = dct['via']
         else:
             self.login_context = self.context
@@ -923,7 +926,7 @@ class Connection(ansible.plugins.connection.ConnectionBase):
         self.close()
 
         inventory_name, stack = self._build_stack()
-        if self._play_context.become:
+        if self.become:
             stack = stack[:-1]
 
         worker_model = ansible_mitogen.process.get_worker_model()
@@ -1129,6 +1132,6 @@ class Connection(ansible.plugins.connection.ConnectionBase):
         self.get_chain().call(
             ansible_mitogen.target.transfer_file,
             context=self.binding.get_child_service_context(),
-            in_path=in_path,
-            out_path=out_path
+            in_path=ansible_mitogen.utils.unsafe.cast(in_path),
+            out_path=ansible_mitogen.utils.unsafe.cast(out_path)
         )
